@@ -1,7 +1,11 @@
+import logging
+
 import sqlalchemy
 import sqlalchemy.orm
 import zope.sqlalchemy
 
+logger = logging.getLogger('khufu_sqlalchemy')
+
 SQLALCHEMY_URL = 'sqlalchemy.url'
 DBSESSION = 'khufu.dbsession'
 DBSESSION_ENGINE = 'khufu.dbengine'
@@ -21,11 +25,25 @@ def _setup_factory(registry):
         engine = settings[DBSESSION_ENGINE]
         factory = sqlalchemy.orm.sessionmaker(
             bind=engine, extension=zope.sqlalchemy.ZopeTransactionExtension())
+        factory = sqlalchemy.orm.scoped_session(factory)
         settings[DBSESSION_FACTORY] = factory
         return factory
 
     url = settings[SQLALCHEMY_URL]
-    engine = settings[DBSESSION_ENGINE] = sqlalchemy.create_engine(url)
+    kwargs = {}
+    if url.startswith('sqlite:'):
+        # dealing with a problem with closed connections
+        # https://github.com/Pylons/pyramid/issues/174
+        import pkg_resources
+        pkg = pkg_resources.get_distribution('SQLAlchemy<0.7dev')
+        if pkg != None:
+            logger.warn('Working around bug with connection pooling and '
+                        'sqlite - '
+                        'https://github.com/Pylons/pyramid/issues/174')
+            from sqlalchemy.pool import NullPool
+            kwargs['poolclass'] = NullPool
+    engine = settings[DBSESSION_ENGINE] = \
+        sqlalchemy.create_engine(url, **kwargs)
     return _setup_factory(registry)
 
 
@@ -67,6 +85,13 @@ class _DBSessionFinder(object):
 
         if create:
             environ[DBSESSION] = request.registry.settings[DBSESSION_FACTORY]()
+
+            def close(request, dbsession=environ[DBSESSION]):
+                try:
+                    dbsession.close()
+                except Exception, ex:
+                    logger.warning(str(ex))
+            request.add_finished_callback(close)
             return environ[DBSESSION]
 
         return None
